void mc_transmit(int device, int* output[5])
{
   int ii;
   
   i2c_start();                  //begin transmission
   i2c_write(device);            //select address of device to communicate with
   for(ii=0;ii<5;   ii++)
   {
      i2c_write(output[ii]);
   }
   i2c_stop();
   delay_us(10);
}

void set_mc_abspos(int device,  int32 position)
{
   int* output[5] = {0,0,0,0,0};
   int ii;
   
   output[0] = 2;
   for(ii=0;ii<4;ii++)
   {
      output[ii+1] = (int8)(position>>(8*(ii)));
   }
   mc_transmit(device,output);
}

void set_mc_vel(int device,  signed int32 velocity)
{
   int* output[5] = {0,0,0,0,0};
   int ii;
   
   output[0] = 3;
   for(ii=0;ii<4;ii++)
   {
      output[ii+1] = (int8)(velocity>>(8*(ii)));
   }
   mc_transmit(device,output);
}

void mc_pos(int device)
{
   int* output[5] = {1,0,0,0,0};
   mc_transmit(device,output);
}

void mc_vel(int device)
{
   int* output[5] = {1,1,0,0,0};
   mc_transmit(device,output);
}

void mc_go()
{
   output_high(pin_a5);
   delay_us(10);
   output_low(pin_a5);
}

void mc_resetpos(int device)
{
   int* output[5] = {10,0,0,0,0};
   mc_transmit(device,output);
}

void set_mc_kp(int device,  int32 kp)
{
   int* output[5] = {4,0,0,0,0};
   int ii;
   for(ii=0;ii<4;ii++)
   {
      output[ii+1] = (int8)(kp>>(8*(ii)));
   }
   mc_transmit(device,output);
}

void set_mc_ki(int device,  int32 ki)
{
   int* output[5] = {5,0,0,0,0};
   int ii;
   for(ii=0;ii<4;ii++)
   {
      output[ii+1] = (int8)(ki>>(8*(ii)));
   }
   mc_transmit(device,output);
}

void set_mc_kd(int device,  int32 kd)
{
   int* output[5] = {6,0,0,0,0};
   int ii;
      for(ii=0;ii<4;ii++)
   {
      output[ii+1] = (int8)(kd>>(8*(ii)));
   }
   mc_transmit(device,output);
}

signed int32 mc_request(int device, int datatype)
{
   int ii;
   int input[4] = {0,0,0,0};
   
   for(ii=0;ii<4;ii++)              //change this back
   {
      i2c_start ();                       //begin communication
      i2c_write (device);                 //send slave address
      i2c_write (datatype-ii);
      i2c_start ();                       //send repeated start command to begin read cycle
      i2c_write (device+1);               //add 1 to the address to send a write bit
      input[ii] = i2c_read(0);            //read requested information from the slave
      i2c_stop ();                        //terminate communication
   }
   return ((int32)input[0]|((int32)input[1]<<8)|((int32)input[2]<<16)|((int32)input[3]<<24));
}

signed int32 mc_get_pos(int device)
{
   int32 position;   
   position = mc_request(device,255);
   return ((signed int32)position);
}

signed int32 mc_get_vel(int device)
{
   int32 velocity;   
   velocity = mc_request(device,251);
   return ((signed int32)velocity);
}

signed int32 mc_get_targetpos(int device)
{
   int32 targetposition;   
   targetposition = mc_request(device,247);
   return ((signed int32)targetposition);
}

signed int32 mc_get_targetvel(int device)
{
   int32 targetvelocity;   
   targetvelocity = mc_request(device,243);
   return ((signed int32)targetvelocity);
}
