#include <18f4520.h>
#fuses HS,NOLVP,NOWDT,NOPROTECT
#use delay(clock=20000000)
#use I2C(FAST, SCL=PIN_C3, SDA=PIN_C4, FORCE_HW)         //Set up Hardware I2C
#use rs232(baud=19200, UART1)                            //Set up PIC UART on RC6 (tx) and RC7 (rx)
#include "MotorControllerFunctions.c"                  //Motor Controller function calls
 
 
// Set up data_tx (transmit values), data_rx (recieve values)
int8 data_tx, data_rx[11] = {0,0,0,0,0,0,0,0,0,0,0};

//Counting Varibles
signed int ii = 0;
int jj = 0;

//Device Name
int Device = 0;

//Temporary variables
int32 Send = 0;
int32 Input=0;


void write()                                             //For writing to MATLAB
{
   for(jj=0;jj<4;jj++)
   {
      data_tx = ((int8)(Send>>(8*jj)));                  //Breaks variables to send to MATLAB into readable Bytes
      printf("%u\n", data_tx);                           //Sends one Byte at a time
   }
}


void readincoming()                                      //Any incoming data is dealt with in this function
{
   data_rx[ii] = fgetc();                                //Read in recieved value from buffer
   if((data_rx[ii]>96)&&(ii>1))                          //Ascii characters above 96 (lowercase letters) are used to recieve values from the Slave
   {
      switch(data_rx[ii])
      {
         case 112:                                       //"p" retrieves position
            Send = mc_get_pos(Device);
            write();
         break;
         case 114:                                       //"r" to reset position
            mc_resetpos(Device);
            mc_go();
         break;
         case 118:                                       //"v" retrieves velocity
            Send = (mc_get_vel(Device)+(1<<31));
            write();
         break;
      }
      ii=-1;                                             //Negative 1 counters ii++ later in function
   } 
   else if ((data_rx[ii]>70)&&(ii>1))                    //Capital Ascii characters are used to set vales to the Slave
   {
      data_rx[2] = data_rx[ii];                          //Ensures no read errors on MATLAB communication 
      ii = 2;
   }
   else if(data_rx[ii]<65)                               //Normalizing Ascii values 1-9 to decimal values 1-9
   {
      data_rx[ii] = data_rx[ii]-48;
   }
   else                                                  //Normalizing Ascii values A-E to decimal values 10-15
   {
      data_rx[ii] = data_rx[ii]-55;
   }
   ii++;                                                 //Increment value being referenced
   if(ii==2)                                             //After recieving the address of the slave, convert from hex to binary
   {
      Device = ((data_rx[0]<<4)|data_rx[1]);
   }
   if(ii>10)                                             //Enter after all hex values recieved
   {
      Input = 0;                                         //Zero out Input variable
      for(jj=0;jj<8;jj++)                                //Convert from eight hex values to one 32 bit binary value
      {
         Input = ((Input<<4)|(data_rx[3+jj]));
      }
      switch(data_rx[2])                                 //Call appropriate functions to carry out actions
      {
         case 73:                                        //"I" for integral constant
            set_mc_ki(Device,(int32)(Input));
            mc_go();
         break;
         case 75:                                        //"K" for proportional constant
            set_mc_kp(Device,(int32)(Input));
            mc_go();
         break;
         case 80:                                        //"P" for position
            mc_pos(Device);
            mc_go();
            set_mc_abspos(Device,(signed int32)(Input-(1<<31)));
            mc_go();
         break;
         case 82:                                        //"R" for derivative constant 
            set_mc_kd(Device,(int32)(Input));
            mc_go();
         break;
         case 86:                                        //"V" for velocity
            mc_vel(Device);
            mc_go();
            set_mc_vel(Device,(signed int32)(Input-(1<<31)));
            mc_go();
         break;
      }
      ii = 0;                                            //Reset counting variable
   }
}
 
 
void main()
{
   
   while (TRUE)                                          //PIC will always watch for data
   {
      if (kbhit())                                       //If PIC senses data pushed to serial buffer
      {
         readincoming();                                 //Call function to handle incoming data
      }
   }
}
