#include <18f4520.h>
#device high_ints=TRUE                                                  //Enable high priority interrupts
#fuses HS,NOLVP,NOWDT,NOPROTECT
#use delay(clock=40000000)                                              //Numbers are preset for 40 mHz clock
#use i2c(SLAVE, FAST, SCL=PIN_C3, SDA=PIN_C4, address=0x80, FORCE_HW)   //Using hardware I2C, defaults to address 0x80 or device 1 

//Encoder Variables
int32 TotalCount = 1<<31;
int16 Count0 = 0, Count1 = 0;
int16 Last0 = 0, Last1 = 0;

//PID vars
int32 error = 0;
signed int32 p_error = 0;
signed int32 i_error = 0;
signed int32 d_error = 0;
signed int32 error_old = 0;
signed int32 pid_error = 0;
int32 kp = 16000;
int32 ki = 0;
int32 kd = 0;
signed int16 Duty = 511;
int normalize = 16;
int i_check = 0;
int i_spacing = 10;

//Velocity Variables
signed int32 Velocity = 0;

//Tracking Variables
int AngleOrVel = 0;
signed int32 TargetVelocity = 0;
int32 TargetPosition = (1<<31);

//I2C variables
int State;                                            //State of Transmission
int32 Number32Bit = 0;                                //Stores bumbers converted to 32 bit
int Number[4] = {0,0,0,0};                            //Stores 4 bytes of 32 bit numbers 
int DataType = 0;                                     //Type of operation being preformed


void runsetup()
{ 
   I2C_SlaveAddr(128|((int)input(pin_B4)<<1)|((int)input(pin_B5)<<2)|((int)input(pin_E2)<<3)|((int)input(pin_E1)<<4));//setting internal address see data sheet for truth table
   
   setup_timer_0(RTCC_EXT_L_TO_H | RTCC_DIV_1);       //external input from quadrature encoder 0
   setup_timer_1(T1_EXTERNAL | T1_DIV_BY_1);          //external input from quadrature encoder 1
   setup_timer_2(T2_DIV_BY_1, 255, 1);                //for PWM at 20KHz, out of audible range
   setup_timer_3(T3_INTERNAL|T3_DIV_BY_1);            //for counting time, increments every .2us and rolls over 76.3 times per sec
   enable_interrupts(INT_SSP);                        //enable I2C interrupt
   enable_interrupts(INT_EXT);                        //enable "go" interrupt on pin C7
   enable_interrupts(global);
   
   setup_ccp2(ccp_pwm);                               //setup PWM

   set_timer0(0);                                     //clear starting point of timers
   set_timer1(0);
   
   set_PWM2_duty(511);                                //motor starts at 0 velocity
      
}
void PID()                                            //Proportional-Integral-Derivative control
{
   switch(AngleOrVel)                                 //0 for angle tracking, 1 for velocity tracking
   {
      case 0:
         if(TargetPosition==TotalCount)               //Exactly on target
         {
            error=0;
            Duty = 511;                               //Stop motor
         }
         else if(TargetPosition>TotalCount)
         {
            error = (TargetPosition-TotalCount);      //Current error
            p_error = ((error*kp)>>normalize);        //Calculating proportional error 
            if (p_error>512)
            {
               Duty = 1023;                           //Motor saturated
            }
            else
            {
               d_error = (error-error_old)*kd;        //Calculating derivative error 
               if (i_check>=i_spacing)                //Spaces out integral calculations to avoid premature integral buildup
               {
                  i_error += error*ki;                //Calculating integral error
                  i_check = 0;
               }
               else
               {
                  i_check++;
               }
               pid_error = p_error+(i_error>>normalize)+d_error;  //PID combined error
      
               if (pid_error>511)                     //PID error will saturate motor
               {
                  if (i_check == 0)
                  {
                     i_error -= error*ki;             //Anti-windup action taken
                  }
                  Duty = 1023;                        //Motor is saturated
               }
               else if(pid_error>(-512))              //PID between -512 and 511 does not saturate motor
               {
                  Duty = 511+pid_error;               //Gives non-saturated duty
               }
               else                                   //PID error less than -512
               {
                  Duty  = 0;                          //Motor saturated
               }
               error_old = error;                     //Remember preveous error for derivative calculation
            }
         }
         else                                         //Same as above but for negative error
         {
            error = (TotalCount-TargetPosition);      //Current error
            p_error = ((error*kp)>>normalize);        //Calculating proportional error
            if (p_error>512)
            {
               Duty = 0;                              //Motor saturated
            }
            else
            {
               d_error = (error-error_old)*kd;        //Calculating derivative error 
               
               if (i_check>=i_spacing)                //Spaces out integral calculations to avoid premature integral buildup
               {
                  i_error -= error*ki;                //Calculating integral error
                  i_check = 0;
               }
               else
               {
                  i_check++;
               }
               pid_error = p_error+(i_error>>normalize)+d_error;  //PID combined error
      
               if (pid_error>511)
               {
                  if (i_check == 0)                   //PID error will saturate motor
                  {
                     i_error += error*ki;             //Anti-windup action taken
                  }
                  Duty = 0;                           //Motor saturated
               }
               else if (pid_error>(-512))             //PID between -512 and 511 does not saturate motor
               {
                  Duty = 512-pid_error;               //Gives non-saturated duty
               }
               else
               {
                  Duty = 1023;                        //PID error less than -512
               }
               error_old = error;                     //Remember preveous error for derivative calculation
            }
         }
      break;
      case 1:                                         //Velocity tracking
         Duty+=TargetVelocity-Velocity;               //Adjust Duty to trak the velocity         
            if (Duty>1023)                               //Saturated Motor
            {
               Duty = 1023;
               Duty-=TargetVelocity-Velocity;
            }
            else if (Duty<0)                             //Saturated Motor
            {
               Duty = 0;
               Duty-=TargetVelocity-Velocity;
            }
         break;
   break;
   }
   set_PWM2_duty(Duty);                               //Adjust PWM in response to PID control
}

void cleartrackingvars()                              //clears all tracking variables on each new command
{
   Duty = 511;                                        //Motors stop
   error = 0;                                         //Error variables set to 0
   p_error = 0;
   i_error = 0;
   d_error = 0;
   error_old = 0;
   pid_error = 0;
}


void updatestatus()                                   //Calculates position and velocity based on encoder counts
{
   Count0 = get_timer0();                             //Up counts
   Count1 = get_timer1();                             //Down counts
   
   Velocity = ((signed int32)(Count0 - Last0))-((signed int32)(Count1 - Last1));    //Current Velocity
   
   disable_interrupts(global);                        //Disable interrupts temporarily to prevent inacurate position calcultaion
   TotalCount += ((int16)(Count0 - Last0));           //Increment up counts
   Last0 = Count0;
   
   TotalCount -= ((int16)(Count1 - Last1));           //Increment down counts
   Last1 = Count1;       
   enable_interrupts(global);                         //Re-enable interrupts

   PID();                                             //Track using PID
}
void updatetracking()                                 //Updates variables on external signal on line B0 
{
   switch(DataType)
   {
      case 1:                                         //Position or Velocity select
         AngleOrVel = (int8)Number32Bit;
      break;
      case 2:                                         //Target position update
         TargetPosition = (1<<31)+(signed int32)Number32Bit;
      break;
      case 3:                                         //Target velocity update
         TargetVelocity = (signed int32)Number32Bit;
      break;
      case 4:                                         //Proportionality constant update
         kp = (int16) Number32Bit;
      break;
      case 5:                                         //Integral constant update
         ki = (int16) Number32Bit;
      break;
      case 6:                                         //Derative constant update
         kd = (int16) Number32Bit;
      break;
      case 10:                                        //Reset position variable
         TotalCount = 1<<31;
         TargetVelocity = 0;
         TargetPosition = 1<<31;
      break;
   }
   cleartrackingvars();                               //Clear tracking variables after new data is received
}

void recombine()                                      //Turns 4 one byte numbers into one 32 bit number
{
   Number32Bit = ((int32)Number[0]|(int32)Number[1]<<8|(int32)Number[2]<<16|(int32)Number[3]<<24);
   delay_us(1);                                       //Prevents overflow
}

#INT_EXT high
void motorgo()                                        //Begins latest tracking information
{
   updatetracking();
}

#INT_SSP high
void ssp_interupt ()                                  //I2C interrupt
{
   disable_interrupts(global);                        //Disable interrupts necessary to prevent exit during transmission
   state = i2c_isr_state();
   if(state < 0x80)                                   //Master is sending data
   {
      if(state == 0)                                  //First received byte is address
      {
      }
      if(state == 1)                                  //Second received byte is address
      {
         DataType = i2c_read();
      }
      if(state >1)                                    //Additional received bytes are data
      {
         Number[state-2] = i2c_read();
      }
      if (state==5)                                   //Combine 4 one byte numbers into one 32 bit number
      {
      recombine();
      }
   }
   if(state >= 0x80)                                  //Master is requesting data
   {
      switch(DataType)                                //Slices data to send to master
      {
         case 255:
            i2c_write((int8)(TotalCount));
         break;
         case 254:
            i2c_write((int8)(TotalCount>>8));
         break;
         case 253:
            i2c_write((int8)(TotalCount>>16));
         break;
         case 252:
            i2c_write((int8)(TotalCount>>24));
         break;
         case 251:
            i2c_write((int8)(Velocity));
         break;
         case 250:
            i2c_write((int8)(Velocity>>8));
         break;
         case 249:
            i2c_write((int8)(Velocity>>16));
         break;
         case 248:
            i2c_write((int8)(Velocity>>24));
         break;
         case 247:
            i2c_write((int8)(TargetPosition));
         break;
         case 246:
            i2c_write((int8)(TargetPosition>>8));
         break;
         case 245:
            i2c_write((int8)(TargetPosition>>16));
         break;
         case 244:
            i2c_write((int8)(TargetPosition>>24));
         break;
         case 243:
            i2c_write((int8)(TargetVelocity));
         break;
         case 242:
            i2c_write((int8)(TargetVelocity>>8));
         break;
         case 241:
            i2c_write((int8)(TargetVelocity>>16));
         break;
         case 240:
            i2c_write((int8)(TargetVelocity>>24));
         break;
      }
   }
   enable_interrupts(global);                         //Re-enable interrupts
}

void main() 
{
   runsetup();
   while (TRUE) 
   {
      while(get_timer3()>2475)                        //Enters main function every 250us or 4000 times per second
      {
         set_timer3(0);                               //Reset timer 3
         updatestatus();
      }
   }
}
